/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.ipod;

import java.io.File;

import java.util.Hashtable;
import java.util.Properties;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.funambol.syncclient.common.DeviceTools;
import com.funambol.syncclient.common.logging.Logger;

import com.funambol.syncclient.spdm.DMException;
import com.funambol.syncclient.spdm.ManagementNode;
import com.funambol.syncclient.spdm.SimpleDeviceManager;

import com.funambol.syncclient.spds.AuthenticationException;
import com.funambol.syncclient.spds.SyncManager;
import com.funambol.syncclient.spds.UpdateException;
import com.funambol.syncclient.spds.event.SyncEvent;
import com.funambol.syncclient.spds.event.SyncItemEvent;
import com.funambol.syncclient.spds.event.SyncItemListener;
import com.funambol.syncclient.spds.event.SyncListener;
import com.funambol.syncclient.spds.event.SyncSourceEvent;
import com.funambol.syncclient.spds.event.SyncSourceListener;
import com.funambol.syncclient.spds.event.SyncStatusEvent;
import com.funambol.syncclient.spds.event.SyncStatusListener;
import com.funambol.syncclient.spds.event.SyncTransportEvent;
import com.funambol.syncclient.spds.event.SyncTransportListener;

import com.funambol.syncclient.ipod.utils.Constants;
import com.funambol.syncclient.ipod.utils.Language;

/**
 * The main window for the IPod Client GUI.
 *
 * @version $Id: MainWindow.java,v 1.11 2007/03/27 10:19:47 luigiafassina Exp $
 */
public class MainWindow extends JFrame
implements ActionListener, Constants, SyncItemListener, SyncListener,
           SyncSourceListener, SyncStatusListener, SyncTransportListener {

    // --------------------------------------------------------------- Constants
    public static final String DM_VALUE_PATH          = "spds/syncml"          ;
    public static final String DM_VALUE_CONTACT_PATH  = "spds/sources/contact" ;
    public static final String DM_VALUE_CALENDAR_PATH = "spds/sources/calendar";
    public static final String DM_VALUE_NOTE_PATH     = "spds/sources/note"    ;

    public static final String PROP_CHARSET           = "spds.charset";
    public static final String PROP_WD                = "wd"          ;

    public static final String VALUE_UTF8             = "UTF8"        ;

    public static final String ROOT_DIRECTORY         = "config"      ;

    // ------------------------------------------------------------ Private data

    /**
     * If true there was a SyncError event into synchronization process, false
     * otherwise
     */
    private boolean syncError = false;

    /**
     * DM root node
     */
    private ManagementNode rootNode = null;

    /**
     * DM values from DM_VALUE_PATH
     */
    private Hashtable syncmlValues = null;

    /**
     * DM values from DM_VALUE_CONTACT_PATH
     */
    private Hashtable xmlContactValues = null;

    /**
     * DM values from DM_VALUE_CALENDAR_PATH
     */
    private Hashtable xmlCalendarValues = null;

    /**
     * DM values from DM_VALUE_NOTE_PATH
     */
    private Hashtable xmlNoteValues = null;

    /**
     * The root directory
     */
    private String rootDirectory = null;

    /**
     * The source directory
     */
    private String sourceDirectory = "";

    /**
     * Status label: contains the status of synchronization process
     */
    private JLabel jlStatus;
    private JButton btSync;
    private MenuBar menuBar;

    // ---------------------------------------------------------- Public methods
    public static void main(String[] args) {
        MainWindow mw = new MainWindow();
        mw.setVisible(true);
    }

    public MainWindow() {
        super();
        Language.init();
        createAndShowMainGUI();

        Properties props = System.getProperties();
        rootDirectory = System.getProperty(PROP_WD);

        if (System.getProperty(SimpleDeviceManager.PROP_DM_DIR_BASE) == null) {
            props.put(SimpleDeviceManager.PROP_DM_DIR_BASE   ,
                      buildPath(rootDirectory, ROOT_DIRECTORY)
            );
            System.setProperties(props);
        }
        props = System.getProperties();
        props.put(PROP_CHARSET, VALUE_UTF8);
        System.setProperties(props);

        sourceDirectory = buildPath(rootDirectory,sourceDirectory);

        //
        // --- Device Manager ---
        //
        rootNode = SimpleDeviceManager.getDeviceManager().getManagementTree();

        checkDeviceID();
        loadConfiguration();
    }

    /**
     * Create and show the main GUI.
     */
    public void createAndShowMainGUI() {
        try {
            UIManager.setLookAndFeel(lookAndFeel);
        } catch(Exception e) {
            e.printStackTrace();
        }

        //
        // Set up the window.
        //
        getContentPane().setLayout(new BorderLayout(1,1));
        setTitle(Language.getMessage(Language.LABEL_TITLE_MAINWINDOW));
        setIconImage(Toolkit.getDefaultToolkit().getImage(FRAME_ICONNAME));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(280,362);
        setLocationRelativeTo(null);
        setResizable(false);
        getContentPane().setBackground(new Color(236,233,216));

        JPanel mainPanel = new JPanel(null);
        mainPanel.setBackground(Color.WHITE);
        getContentPane().add(mainPanel, BorderLayout.CENTER);

        JLabel lab = new JLabel(new ImageIcon(FRAME_LOGONAME));
        lab.setBounds(72, 30, 128, 128);
        mainPanel.add(lab);

        btSync = new JButton(Language.getMessage(Language.BT_SYNC));
        btSync.setFont(new Font("Microsoft Sans Serif", 0, 13));
        btSync.setBounds(81, 233, 112, 24);
        btSync.setBackground(Color.WHITE);
        btSync.addActionListener(this);

        mainPanel.add(btSync);

        mainPanel.setBorder(BorderFactory.createRaisedBevelBorder());

        //
        // Label for show the status of synchronization process
        //
        jlStatus = new JLabel();
        jlStatus.setFont(font);
        jlStatus.setPreferredSize(new Dimension(260, 25));
        getContentPane().add(jlStatus, BorderLayout.SOUTH);

        menuBar = new MenuBar(this);
        setJMenuBar(menuBar);
    }

    /**
     * This method is call only when the button "Synchronization" is pushed.
     */
    public void actionPerformed(ActionEvent e) {
        if (checkSyncSettings()) {
          runSynchronization();
        } else {
            setStatusMessage(Language.getMessage(Language.STATUS_NOT_SYNC_REQUIRED));
        }
    }

    /**
     * Check if there is least one source to synchronize
     *
     * @return true if there is least one source to sync, otherwise false
     */
    private boolean checkSyncSettings() {
        String contactSM = (String)xmlContactValues.get(PARAM_SYNCMODE);
        String calendarSM = (String)xmlCalendarValues.get(PARAM_SYNCMODE);

        if (contactSM.equalsIgnoreCase(SYNC_NONE) &&
            calendarSM.equalsIgnoreCase(SYNC_NONE)  ) {
            return false;
        }
        return true;
    }

    public void runSynchronization() {
        File f = null;
        if ((f = getFileLog()) != null) {
            f.delete();
        }
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        setStatusMessage(Language.getMessage(Language.STATUS_SYNC_BEGIN));

        new Thread(new Runnable() {
            public void run() {

                //
                // Disable each ui control by which a user can start a sync
                //
                btSync.setEnabled(false);
                menuBar.setEnableSyncMenuItem(false);

                //
                // Do the sync
                //
                synchronize();

                //
                // Enable previously disabled controls
                //
                btSync.setEnabled(true);
                menuBar.setEnableSyncMenuItem(true);
            }
        }).start();
    }

    /**
     * Start the synchronization process based on the configuration parameters
     */
    private void synchronize() {
        checkDeviceID();
        try {

            SyncManager syncManager = SyncManager.getSyncManager("");

            syncManager.addSyncItemListener     (this);
            syncManager.addSyncListener         (this);
            syncManager.addSyncSourceListener   (this);
            syncManager.addSyncStatusListener   (this);
            syncManager.addSyncTransportListener(this);

            syncManager.sync();

        } catch (AuthenticationException e) {
            setStatusMessage(Language.getMessage(Language.ERROR_AUTH));
        } catch (UpdateException e) {
            setStatusMessage(Language.getMessage(Language.ERROR_SERVER_GENERIC));
        } catch (Exception e) {
            setStatusMessage(Language.getMessage(Language.ERROR_CONNECT));
        }

        if (syncError) {
            setStatusMessage(Language.getMessage(Language.ERROR_SYNC));
        }

        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    /**
     * Show the message for the status of synchronization process
     *
     * @param msg the message to show
     */
    private void setStatusMessage(String msg) {
        jlStatus.setText(msg);
        jlStatus.setAutoscrolls(true);
        jlStatus.setHorizontalAlignment(SwingConstants.CENTER);
    }

    /**
     * Saves the specified configuration parameters
     * by passing them to the DeviceManager
     *
     * @param values a list of configuration values
     */
    public void writeSyncSettings(Hashtable values) {

        try {
            xmlContactValues.put(PARAM_SYNCSOURCEDRIVE, values.get(PARAM_SYNCSOURCEDRIVE));
            xmlContactValues.put(PARAM_SYNCMODE, values.get(PARAM_SYNCCONTACT));

            xmlCalendarValues.put(PARAM_SYNCSOURCEDRIVE, values.get(PARAM_SYNCSOURCEDRIVE));
            xmlCalendarValues.put(PARAM_SYNCMODE, values.get(PARAM_SYNCCALENDAR));

            xmlNoteValues.put(PARAM_SYNCSOURCEDRIVE, values.get(PARAM_SYNCSOURCEDRIVE));
            xmlNoteValues.put(PARAM_SYNCMODE, values.get(PARAM_SYNCNOTE));

            //
            // Set Logging level
            //
            syncmlValues.put (PARAM_LOGLEVEL,values.get(PARAM_LOGLEVEL));
            rootNode.setValue(DM_VALUE_PATH, PARAM_LOGLEVEL, values.get(PARAM_LOGLEVEL));

            //
            // Store source drive
            //
            rootNode.setValue(DM_VALUE_CONTACT_PATH,
                              PARAM_SYNCSOURCEDRIVE,
                              values.get(PARAM_SYNCSOURCEDRIVE)
            );
            rootNode.setValue(DM_VALUE_CALENDAR_PATH,
                              PARAM_SYNCSOURCEDRIVE ,
                              values.get(PARAM_SYNCSOURCEDRIVE)
            );
            rootNode.setValue(DM_VALUE_NOTE_PATH   ,
                              PARAM_SYNCSOURCEDRIVE,
                              values.get(PARAM_SYNCSOURCEDRIVE)
            );

            //
            // Store sync mode
            //
            rootNode.setValue(DM_VALUE_CONTACT_PATH,
                              PARAM_SYNCMODE       ,
                              values.get(PARAM_SYNCCONTACT)
            );
            rootNode.setValue(DM_VALUE_CALENDAR_PATH,
                              PARAM_SYNCMODE        ,
                              values.get(PARAM_SYNCCALENDAR)
            );
            rootNode.setValue(DM_VALUE_NOTE_PATH,
                              PARAM_SYNCMODE    ,
                              values.get(PARAM_SYNCNOTE)
            );

        } catch (DMException e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_DEVICE_MANAGER));
                Logger.error(e.getMessage());
            }
        } catch (Exception e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_WRITE_SYNCSET));
                Logger.error(e.getMessage());
            }
        }
    }

    /**
     * Saves the specified configuration parameters
     * by passing them to the DeviceManager
     *
     * @param values a list of configuration values
     */
    public void writeRemoteSettings(Hashtable values) {

        try {
            xmlContactValues.put(PARAM_SYNCSOURCEURI, values.get(PARAM_SOURCEURICONTACT));
            xmlCalendarValues.put(PARAM_SYNCSOURCEURI,values.get(PARAM_SOURCEURICALENDAR));
            xmlNoteValues.put(PARAM_SYNCSOURCEURI,values.get(PARAM_SOURCEURINOTE));

            rootNode.setValue(DM_VALUE_CONTACT_PATH            ,
                              PARAM_SYNCSOURCEURI              ,
                              values.get(PARAM_SOURCEURICONTACT)
            );
            rootNode.setValue(DM_VALUE_CALENDAR_PATH            ,
                              PARAM_SYNCSOURCEURI               ,
                              values.get(PARAM_SOURCEURICALENDAR)
            );
            rootNode.setValue(DM_VALUE_NOTE_PATH            ,
                              PARAM_SYNCSOURCEURI           ,
                              values.get(PARAM_SOURCEURINOTE)
            );
        } catch (DMException e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_DEVICE_MANAGER));
                Logger.error(e.getMessage());
            }
        } catch (Exception e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_WRITE_REMOTESET));
                Logger.error(e.getMessage());
            }
        }
    }

    /**
     * Saves the specified configuration parameters
     * by passing them to the DeviceManager
     *
     * @param values a list of configuration values
     */
    public void writeCommunicationSettings(Hashtable values) {
        try {
            syncmlValues.put(PARAM_SYNCMLURL, values.get(PARAM_SYNCMLURL));
            syncmlValues.put(PARAM_USERNAME , values.get(PARAM_USERNAME ));
            syncmlValues.put(PARAM_PASSWORD , values.get(PARAM_PASSWORD ));
            syncmlValues.put(PARAM_DEVICEID , values.get(PARAM_DEVICEID ));

            rootNode.setValue(DM_VALUE_PATH             ,
                              PARAM_TARGETLOCALURI      ,
                              values.get(PARAM_SYNCMLURL)
            );
            rootNode.setValue(DM_VALUE_PATH             ,
                              PARAM_SYNCMLURL           ,
                              values.get(PARAM_SYNCMLURL)
            ) ;
            rootNode.setValue(DM_VALUE_PATH            ,
                              PARAM_USERNAME           ,
                              values.get(PARAM_USERNAME)
            );
            rootNode.setValue(DM_VALUE_PATH            ,
                              PARAM_PASSWORD           ,
                              values.get(PARAM_PASSWORD)
            );
            rootNode.setValue(DM_VALUE_PATH            ,
                              PARAM_DEVICEID           ,
                              values.get(PARAM_DEVICEID)
            );

        } catch (DMException e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_DEVICE_MANAGER));
                Logger.error(e.getMessage());
            }
        } catch (Exception e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_WRITE_COMMUNICATIONSET));
                Logger.error(e.getMessage());
            }
        }

        checkDeviceID();
    }

    public Hashtable getXmlContactValues() {
        return this.xmlContactValues;
    }

    public Hashtable getXmlCalendarValues() {
        return this.xmlCalendarValues;
    }

    public Hashtable getXmlNoteValues() {
        return this.xmlNoteValues;
    }

    public Hashtable getSyncmlValues() {
        return this.syncmlValues;
    }

    public File getFileLog() {
        File f = new File(rootDirectory + File.separator + "sync.log");
        if (f.exists()) {
            return f;
        }
        return null;
    }

    /**
     * Closes the window and stops the application
     */
    public void exit() {
        setVisible(false);
        dispose();

        if (Logger.isLoggable(Logger.INFO)) {
            Logger.info(Language.getMessage(Language.LOGGING_STOPPED));
        }
        System.exit(0);
    }

    /**
     * Fired when the synchronization process start.
     *
     * @param syncEvent the synchronization event
     */
    public void syncBegin(SyncEvent syncEvent) {
        syncError = false;
        if (Logger.isLoggable(Logger.INFO)) {
            Logger.info(syncEvent.getDate() + ", " +
                        Language.getMessage(Language.LOG_SYNC_BEGIN)
            );
        }
    }

    /**
     * Fired when the synchronization process end.
     *
     * @param syncEvent the synchronization event
     */
    public void syncEnd(SyncEvent syncEvent) {
        if (Logger.isLoggable(Logger.INFO)) {
            Logger.info(syncEvent.getDate() + ", " +
                        Language.getMessage(Language.LOG_SYNC_END)
            );
        }
        setStatusMessage(Language.getMessage(Language.STATUS_SYNC_END));
    }

    /**
     * Fired to notify that the engine encountered a not blocking error
     *
     * @param syncEvent the synchronization event
     */
    public void syncError(SyncEvent syncEvent) {
        syncError = true;
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(syncEvent.getDate() + ", "
                         + Language.getMessage(Language.LOG_SYNC_ERROR)
                         + " - " + Language.getMessage(Language.LOG_SYNCERROR_MSG)
                         + " " + syncEvent.getMessage()
                         + " - " + Language.getMessage(Language.LOG_SYNCCAUSE_MSG)
                         + " " + syncEvent.getCause().getMessage()
            );
        }
    }

    /**
     * Fired to notify that the initialization package was correctly set and
     * precessed.
     *
     * @param syncEvent the synchronization event
     */
    public void sendInitialization(SyncEvent syncEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(syncEvent.getDate() + ", " +
                         Language.getMessage(Language.LOG_SYNC_SENDINIT)
            );
        }
    }

    /**
     * Fired to notify that the modifications package was correctly set and
     * processed
     *
     * @param syncEvent the synchronization event
     */
    public void sendModification(SyncEvent syncEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(syncEvent.getDate() + ", " +
                         Language.getMessage(Language.LOG_SYNC_SENDMOD)
            );
        }
    }

    /**
     * Fired to notify that the final package was correctly set and processed.
     *
     * @param syncEvent the synchronization event
     */
    public void sendFinalization(SyncEvent syncEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(syncEvent.getDate() + ", " +
                         Language.getMessage(Language.LOG_SYNC_SENDFINAL)
            );
        }
    }

    /**
     * Fired to notify that the engine started to send data to the server
     *
     * @param syncTransportEvent the SyncTransportEvent event
     */
    public void sendDataBegin(SyncTransportEvent syncTransportEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCTRANSPORT_SENDDATABEGIN)
                         + " " + syncTransportEvent.getData()
            );
        }
    }

    /**
     * Fired to notify that the engine has sent all data to the server
     *
     * @param syncTransportEvent the SyncTransportEvent event
     */
    public void sendDataEnd(SyncTransportEvent syncTransportEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCTRANSPORT_SENDDATAEND)
                         + " " + syncTransportEvent.getData()
            );
        }
    }

    /**
     * Fired to notify that the engine started to receive data from the server
     *
     * @param syncTransportEvent the SyncTransportEvent event
     */
    public void receiveDataBegin(SyncTransportEvent syncTransportEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCTRANSPORT_RECEIVEDATABEGIN)
                         + " " + syncTransportEvent.getData()
            );
        }
    }

    /**
     * Fired to notify that the engine is receiving data from the server
     *
     * @param syncTransportEvent the SyncTransportEvent event
     */
    public void dataReceived(SyncTransportEvent syncTransportEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCTRANSPORT_RECEIVEDATA)
                         + " " + syncTransportEvent.getData()
            );
        }
    }

    /**
     * Fired to notify that the engine has received all data from the server
     *
     * @param syncTransportEvent the SyncTransportEvent event
     */
    public void receiveDataEnd(SyncTransportEvent syncTransportEvent) {
       if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCTRANSPORT_RECEIVEDATAEND)
                         + " " + syncTransportEvent.getData()
            );
        }
    }

    /**
     * Fired to notify the beginning of the synchronization of a SyncSource
     *
     * @param syncSourceEvent the SyncSource event
     */
    public void syncBegin(SyncSourceEvent syncSourceEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(syncSourceEvent.getDate() + ", "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_BEGIN)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncSourceEvent.getSourceUri()
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SYNCMODE)
                         + " " + syncSourceEvent.getSyncMode()
            );
        }
    }

    /**
     * Fired to notify the end of the synchronization of a SyncSource
     *
     * @param syncSourceEvent the SyncSource event
     */
    public void syncEnd(SyncSourceEvent syncSourceEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(syncSourceEvent.getDate() + ", "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_END)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncSourceEvent.getSourceUri()
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SYNCMODE)
                         + " " + syncSourceEvent.getSyncMode()
            );
        }
    }

    /**
     * Fired to notify that an item addition has been received
     *
     * @param syncItemEvent the SyncItemEvent event
     */
    public void itemAddedByServer(SyncItemEvent syncItemEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCITEM_ADDSERVER)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncItemEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncItemEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify that an item deletion has been received
     *
     * @param syncItemEvent the SyncItemEvent event
     */
    public void itemDeletedByServer(SyncItemEvent syncItemEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCITEM_DELETESERVER)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncItemEvent.getSourceUri()
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncItemEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify that an item update has been received.
     *
     * @param syncItemEvent the SyncItemEvent event
     */
    public void itemUpdatedByServer(SyncItemEvent syncItemEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCITEM_UPDATESERVER)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncItemEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncItemEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify that an item addition has been sent.
     *
     * @param syncItemEvent the SyncItemEvent event
     */
    public void itemAddedByClient(SyncItemEvent syncItemEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCITEM_ADDCLIENT)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncItemEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncItemEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify that an item deletion has been sent.
     *
     * @param syncItemEvent the SyncItemEvent event
     */
    public void itemDeletedByClient(SyncItemEvent syncItemEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCITEM_DELETECLIENT)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncItemEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncItemEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify that an item update has been sent.
     *
     * @param syncItemEvent the SyncItemEvent event
     */
    public void itemUpdatedByClient(SyncItemEvent syncItemEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCITEM_UPDATECLIENT)
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncItemEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncItemEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify a status command to send
     *
     * @param syncStatusEvent the SyncStatusEvent event
     */
    public void statusToSend(SyncStatusEvent syncStatusEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCSTATUS_SEND)
                         + " - " + Language.getMessage(Language.LOG_SYNCSTATUS_CMD)
                         + " " + syncStatusEvent.getCommand()
                         + " - " + Language.getMessage(Language.LOG_SYNCSTATUS_STATUS)
                         + " " + syncStatusEvent.getStatusCode()
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncStatusEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncStatusEvent.getItemKey().getKeyAsString()
            );
        }
    }

    /**
     * Fired to notify a received status command
     *
     * @param syncStatusEvent the SyncStatusEvent event
     */
    public void statusReceived(SyncStatusEvent syncStatusEvent) {
        if (Logger.isLoggable(Logger.DEBUG)) {
            Logger.debug(Language.getMessage(Language.LOG_SYNCSTATUS_RECEIVED)
                         + " - " + Language.getMessage(Language.LOG_SYNCSTATUS_CMD)
                         + " " + syncStatusEvent.getCommand()
                         + " - " + Language.getMessage(Language.LOG_SYNCSTATUS_STATUS)
                         + " " + syncStatusEvent.getStatusCode()
                         + " - "
                         + Language.getMessage(Language.LOG_SYNCSOURCE_SOURCEURI)
                         + " " + syncStatusEvent.getSourceUri()
                         + " - " + Language.getMessage(Language.LOG_SYNCITEM_KEY)
                         + " " + syncStatusEvent.getItemKey().getKeyAsString()
            );
        }
    }
    
    /**
     * Loads DM configuration
     */
    public void loadConfiguration() {
        try {
            syncmlValues      = rootNode.getNodeValues(DM_VALUE_PATH         );
            xmlContactValues  = rootNode.getNodeValues(DM_VALUE_CONTACT_PATH );
            xmlCalendarValues = rootNode.getNodeValues(DM_VALUE_CALENDAR_PATH);
            xmlNoteValues     = rootNode.getNodeValues(DM_VALUE_NOTE_PATH    );

            //
            // Setting default logging properties
            //
            String logLevel = (String)syncmlValues.get(PARAM_LOGLEVEL);
            if(logLevel.equals(LOG_INFO)) {
                Logger.setLevel(Logger.INFO);
            } else if(logLevel.equals(LOG_DEBUG)) {
                Logger.setLevel(Logger.DEBUG);
            }
            Logger.setDefaultLogFile();

        } catch (DMException e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_DEVICE_MANAGER));
                Logger.error(e.getMessage());
            }
        }
    }    

    /**
     * Checks if the device id is not null or empty. If so, a new device id will
     * be created and stored in the dm tree
     */
    public void checkDeviceID() {
        String deviceID = null;
        try {
            deviceID = (String)rootNode.getNodeValue(DM_VALUE_PATH, PARAM_DEVICEID);
            if (deviceID == null || deviceID.length() == 0) {
                deviceID = DeviceTools.createDeviceID(PREFIX_DEVICE_ID, DEFAULT_DEVICE_ID);
                rootNode.setValue(DM_VALUE_PATH, PARAM_DEVICEID, deviceID);
            }
        } catch (DMException e) {
            if (Logger.isLoggable(Logger.ERROR)) {
                Logger.error(Language.getMessage(Language.ERROR_DEVICE_MANAGER));
                Logger.error(e.getMessage());
            }
        }
    }
    
    // --------------------------------------------------------- Private methods

    /**
     * Returns a string representing the path of the specified dir
     * concatenating it with the specified source directory
     *
     * @param sourceDirectory the source directory
     * @param dir the directory
     * @return the directory's path
     */
    private String buildPath(String sourceDirectory, String dir) {
        return sourceDirectory+"/"+dir;
    }

}
