/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.ipod;

import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import com.funambol.syncclient.ipod.utils.Language;
import com.funambol.syncclient.ipod.panels.AboutPanel;
import com.funambol.syncclient.ipod.panels.CommunicationSetPanel;
import com.funambol.syncclient.ipod.panels.LogPanel;
import com.funambol.syncclient.ipod.panels.SyncSetPanel;

/**
 * The menu bar for this GUI.
 *
 * @version $Id: MenuBar.java,v 1.6 2007/03/04 17:18:12 nichele Exp $
 */
public class MenuBar extends JMenuBar implements ActionListener {

    // ------------------------------------------------------------ Private Data
    private Font       font         = new Font("Microsoft Sans Serif", 0, 12);
    private JMenu      menu         = null;
    private JMenuItem  menuItem     = null;
    private JMenuItem  syncMenuItem = null;
    private MainWindow mainWindow   = null;
    private Color      color        = new Color(236,233,216);

    // ------------------------------------------------------------ Constructors
    public MenuBar(MainWindow mainWindow) {

        this.mainWindow = mainWindow;

        setBackground(color);

        //
        // File Menu
        //
        menu = new JMenu(Language.getMessage(Language.MENU_FILE));
        menu.setBackground(color);
        menu.setFont(font);

        menuItem = new JMenuItem(Language.getMessage(Language.MENU_FILE_SYNC));
        menuItem.setFont(font);
        menuItem.setBackground(Color.WHITE);
        menuItem.setActionCommand("SYNCHRONIZATION");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        syncMenuItem = menuItem;

        menu.addSeparator();

        menuItem = new JMenuItem(Language.getMessage(Language.MENU_FILE_EXIT));
        menuItem.setFont(font);
        menuItem.setBackground(Color.WHITE);
        menuItem.setActionCommand("EXIT");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        add(menu);

        //
        // Edit Menu
        //
        menu = new JMenu(Language.getMessage(Language.MENU_EDIT));
        menu.setBackground(color);
        menu.setFont(font);

        menuItem = new JMenuItem(Language.getMessage(Language.MENU_EDIT_COMMUNICATION_SET));
        menuItem.setBackground(Color.WHITE);
        menuItem.setFont(font);
        menuItem.setActionCommand("COMMUNICATIONSET");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem(Language.getMessage(Language.MENU_EDIT_SYNC_SET));
        menuItem.setBackground(Color.WHITE);
        menuItem.setFont(font);
        menuItem.setActionCommand("SYNCSET");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        add(menu);

        //
        // Help Menu
        //
        menu = new JMenu(Language.getMessage(Language.MENU_HELP));
        menu.setBackground(color);
        menu.setFont(font);

        menuItem = new JMenuItem(Language.getMessage(Language.MENU_HELP_LOG));
        menuItem.setBackground(Color.WHITE);
        menuItem.setFont(font);
        menuItem.setActionCommand("VIEWLOG");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menu.addSeparator();

        menuItem = new JMenuItem(Language.getMessage(Language.MENU_HELP_ABOUT));
        menuItem.setBackground(Color.WHITE);
        menuItem.setFont(font);
        menuItem.setActionCommand("ABOUT");
        menuItem.addActionListener(this);
        menu.add(menuItem);

        add(menu);

    }
    
    // ---------------------------------------------------------- Public Methods
    
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("SYNCHRONIZATION")) {
            mainWindow.runSynchronization();
        } else if(e.getActionCommand().equals("COMMUNICATIONSET")) {
            mainWindow.checkDeviceID();
            mainWindow.loadConfiguration();
            CommunicationSetPanel c = new CommunicationSetPanel(mainWindow,true);
            c.show();
        } else if(e.getActionCommand().equals("SYNCSET")) {
            SyncSetPanel ssp = new SyncSetPanel(mainWindow,true);
            ssp.show();
        } else if(e.getActionCommand().equals("VIEWLOG")) {
            LogPanel lp = new LogPanel(mainWindow,true);
            lp.show();
        } else if(e.getActionCommand().equals("ABOUT")) {
            AboutPanel ap = new AboutPanel(mainWindow,true);
            ap.show();
        } else if(e.getActionCommand().equals("EXIT")) {
            mainWindow.exit();
        }

    }
    
    /**
     * Enable/disable the synchronization option of the File menu.
     * @param enabled it must be equals to <code>true</code> to enable the 
     * synchronization option, <code>false</code> to disable
     */
    public void setEnableSyncMenuItem(boolean enabled){
        syncMenuItem.setEnabled(enabled);
    }
    
}