/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.ipod.panels;

import java.util.Hashtable;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.UIManager;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.border.*;
import javax.swing.SwingConstants;

import com.funambol.syncclient.ipod.MainWindow;
import com.funambol.syncclient.ipod.utils.Constants;
import com.funambol.syncclient.ipod.utils.Language;

/**
 * The Communication settings window.
 *
 * @version $Id: CommunicationSetPanel.java,v 1.4 2007/01/11 19:23:47 nichele Exp $
 */
public class CommunicationSetPanel extends JDialog
implements ActionListener, Constants {

    // ------------------------------------------------------------ Private Data
    private Hashtable  htFields = new Hashtable();
    private MainWindow mainWindow ;
    private JPanel     jpOver     ;
    private JPanel     jpUp       ;
    private JLabel     jLabel1    ;
    private JLabel     jLabel2    ;
    private JLabel     jLabel3    ;
    private JLabel     jLabel4    ;
    private JTextField jtServerURL;
    private JTextField jtUsername ;
    private JTextField jtDeviceId ;
    private JPasswordField jpPassword;
    private JButton    btOk    ;
    private JButton    btCancel;

    // ---------------------------------------------------------- Public Methods

    /** Creates new form JDialog */
    public CommunicationSetPanel(MainWindow mainWindow, boolean modal) {
        super(mainWindow, modal);
        this.mainWindow = mainWindow;

        htFields = mainWindow.getSyncmlValues();

        initComponents();
        setCommunicationConfigStored();
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        int xlab  = 18 ;
        int wlab  = 90 ;
        int hlab  = 21 ;
        int xtext = 130;
        int wtext = 222;
        int htext = 21 ;
        int ybt   = 147;
        int wbt   = 88 ;
        int hbt   = 24 ;
        int y     = 26 ;
        int dy    = 24 ;

        jLabel1     = new JLabel();
        jLabel2     = new JLabel();
        jLabel3     = new JLabel();
        jLabel4     = new JLabel();
        jpOver      = new JPanel();
        jpUp        = new JPanel();
        jtServerURL = new JTextField();
        jtUsername  = new JTextField();
        jtDeviceId  = new JTextField();
        jpPassword  = new JPasswordField();
        btOk        = new JButton();
        btCancel    = new JButton();

        try {
            UIManager.setLookAndFeel(lookAndFeel);
        } catch(Exception e) {
            e.printStackTrace();
        }

        //
        // Set up the window.
        //
        getContentPane().setLayout(null);

        setTitle(Language.getMessage(Language.LABEL_TITLE_COMMUNICATION));
        setBackground(Color.WHITE);
        setForeground(Color.WHITE);
        setFont(font);
        setName("commSetDialog");
        setResizable(false);
        setSize(400,214);
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setLocationRelativeTo(null);

        jpOver.setLayout(null);
        jpOver.setBackground(Color.WHITE);
        jpOver.setForeground(Color.WHITE);
        jpOver.setBounds(0, 0, 400, 214);
        getContentPane().add(jpOver);

        //
        // Internal panel for Remote Settings
        //
        jpUp.setLayout(null);
        jpUp.setBounds(8, 12, 371, 132);
        jpUp.setBackground(Color.WHITE);
        jpUp.setBorder(new TitledBorder(
                            new LineBorder(new Color(204,204,204), 1, true),
                            Language.getMessage(Language.LABEL_TITLE_COMMUNICATION),
                            1,
                            0,
                            font,
                            new Color(0,51,204))
        );
        jpUp.setName("commSet");

        jLabel1.setText(Language.getMessage(Language.LABEL_COMM_SERVERURL));
        jLabel1.setFont(font);
        jLabel1.setBounds(xlab, y, wlab, hlab);
        jLabel1.setVerticalAlignment(SwingConstants.CENTER);
        jpUp.add(jLabel1);

        jtServerURL.setText("");
        jtServerURL.setFont(font);
        jtServerURL.setBounds(xtext, 26, wtext, htext);
        jtServerURL.requestFocusInWindow();
        jpUp.add(jtServerURL);

        y += dy;

        jLabel2.setText(Language.getMessage(Language.LABEL_COMM_USERNAME));
        jLabel2.setFont(font);
        jLabel2.setBounds(xlab, y, wlab, hlab);
        jLabel2.setVerticalAlignment(SwingConstants.CENTER);
        jpUp.add(jLabel2);

        jtUsername.setText("");
        jtUsername.setFont(font);
        jtUsername.setBounds(xtext, 50, wtext, htext);
        jpUp.add(jtUsername);

        y += dy;

        jLabel3.setText(Language.getMessage(Language.LABEL_COMM_PASSWORD));
        jLabel3.setBounds(xlab, y, wlab, hlab);
        jLabel3.setFont(font);
        jLabel3.setVerticalAlignment(SwingConstants.CENTER);
        jpUp.add(jLabel3);

        jpPassword.setText("");
        jpPassword.setFont(font);
        jpPassword.setBounds(xtext, 74, wtext, htext);
        jpUp.add(jpPassword);

		y += dy;

        jLabel4.setText(Language.getMessage(Language.LABEL_COMM_DEVICEID));
        jLabel4.setBounds(xlab, y, wlab, hlab);
        jLabel4.setFont(font);
        jLabel4.setVerticalAlignment(SwingConstants.CENTER);
        jpUp.add(jLabel4);

        jtDeviceId.setText("");
        jtDeviceId.setFont(font);
        jtDeviceId.setBounds(xtext, 98, wtext, htext);
        jtDeviceId.setEditable(false);
        jpUp.add(jtDeviceId);

        jpOver.add(jpUp);

        btOk.setText(Language.getMessage(Language.BT_OK));
        btOk.setActionCommand("OK");
        btOk.setName("btOk");
        btOk.setFont(font);
        btOk.setBounds(198, ybt, wbt, hbt);
        btOk.addActionListener(this);

        jpOver.add(btOk);

        btCancel.setText(Language.getMessage(Language.BT_CANCEL));
        btCancel.setActionCommand("CANCEL");
        btCancel.setName("btCancel");
        btCancel.setBounds(292, ybt, wbt, hbt);
        btCancel.setFont(font);
        btCancel.addActionListener(this);
        jpOver.add(btCancel);

        jtServerURL.setNextFocusableComponent(jtUsername);
        jtUsername.setNextFocusableComponent(jpPassword) ;
        jpPassword.setNextFocusableComponent(jtDeviceId) ;
        jtDeviceId.setNextFocusableComponent(btOk)       ;
        btOk.setNextFocusableComponent(btCancel)         ;
        btCancel.setNextFocusableComponent(jtServerURL)  ;
    }

    public void actionPerformed(ActionEvent evt) {
        if (evt.getActionCommand().equals("OK")) {
            htFields.put(PARAM_SYNCMLURL, jtServerURL.getText());
            htFields.put(PARAM_USERNAME , jtUsername.getText());
            htFields.put(PARAM_PASSWORD , new String(jpPassword.getPassword()));
            htFields.put(PARAM_DEVICEID , jtDeviceId.getText());

            mainWindow.writeCommunicationSettings(htFields);
            removeAll();
            setVisible(false);
        } else if (evt.getActionCommand().equals("CANCEL")) {
            removeAll();
            setVisible(false);
        }
    }

    /**
     * Sets the fields with some of the values contained
     * in the specified Hashtables.
     */
    private void setCommunicationConfigStored() {
        jtServerURL.setText((String)htFields.get(PARAM_SYNCMLURL));
        jtUsername.setText ((String)htFields.get(PARAM_USERNAME) );
        jpPassword.setText ((String)htFields.get(PARAM_PASSWORD) );
        jtDeviceId.setText ((String)htFields.get(PARAM_DEVICEID) );
    }

}
