/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.ipod.panels;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.UIManager;
import javax.swing.border.*;
import javax.swing.SwingConstants;

import com.funambol.syncclient.ipod.MainWindow;
import com.funambol.syncclient.ipod.utils.Constants;
import com.funambol.syncclient.ipod.utils.Language;

/**
 * The Logging window.
 *
 * @version $Id: LogPanel.java,v 1.5 2007/03/27 10:17:17 luigiafassina Exp $
 */
public class LogPanel extends JDialog implements ActionListener, Constants {

    // ------------------------------------------------------------ Private Data
    private MainWindow mainWindow ;
    private boolean modal = false ;

    private JButton     jbClose     ;
    private JScrollPane jScrollPane1;
    private JTextArea   jTextArea   ;
    private JPanel      jPanel      ;

    // ---------------------------------------------------------- Public Methods

    /** Creates new form JDialog */
    public LogPanel(MainWindow mainWindow, boolean modal) {
        super(mainWindow, modal);
        this.mainWindow = mainWindow;
        this.modal      = modal     ;

        initComponents();
        getFileLog();
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        jbClose      = new JButton()    ;
        jScrollPane1 = new JScrollPane();
        jTextArea    = new JTextArea()  ;
        jPanel       = new JPanel()     ;

        try {
            UIManager.setLookAndFeel(lookAndFeel);
        } catch(Exception e) {
            e.printStackTrace();
        }

        //
        // Set up the window.
        //
        getContentPane().setLayout(new BorderLayout(1,1));
        setTitle(Language.getMessage(Language.LABEL_TITLE_LOGWINDOW));
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setSize(600,600);
        setLocationRelativeTo(null);
        setFont(font);
        setName("logDialog");
        setResizable(true);

        jTextArea.setEditable(false);
        jTextArea.setFont(font);
        jTextArea.setBackground(Color.WHITE);
        jScrollPane1.setAutoscrolls(true);
        jScrollPane1.setViewportView(jTextArea);
        getContentPane().add(jScrollPane1, BorderLayout.CENTER);

        jPanel.setLayout(new GridBagLayout());
        jPanel.setBackground(Color.WHITE);
        jPanel.setPreferredSize(new Dimension(400, 50));
        getContentPane().add(jPanel, BorderLayout.SOUTH);

        jbClose.setText(Language.getMessage(Language.BT_CLOSE));
        jbClose.setFont(font);
        jbClose.setPreferredSize(new Dimension(88, 23));
        jbClose.setVerticalAlignment(SwingConstants.CENTER);
        jbClose.setHorizontalAlignment(SwingConstants.CENTER);
        jbClose.addActionListener(this);
        jbClose.requestFocus(true);
        jPanel.add(jbClose);
    }

    public void actionPerformed(ActionEvent evt) {
        removeAll();
        setVisible(false);
    }

    /**
     * Read the file with logging to show
     */
    private void getFileLog() {
        try {
            File logfile = mainWindow.getFileLog();
            if (logfile != null) {
                FileReader fr = new FileReader(logfile);
                BufferedReader buf = new BufferedReader(fr);

                String line = null;
                while ((line = buf.readLine()) != null) {
                    jTextArea.append(line);
                    jTextArea.append("\n");
                }
                buf.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

}
